local i18n = QBL.i18n[QBL.locale].lib.Interface

local main_f = QBL.frames.main
local menu_f = QBL.frames.menu

local try_sound = QBL.util.Config.try_sound

-- Export table for Interface.
QBL.lib.Interface = {}

-- Called to initialize a given dropdown menu.
local function init_dropdown(self)
	local info = UIDropDownMenu_CreateInfo()
	info.func = self.onclick
	info.owner = self
	info.tooltipOnButton = true
	info.tooltipTitle, info.tooltipText = self.tiptitle, self.tip
	for k, v in ipairs(self.items) do
		info.text, info.arg1, info.checked = v, k, false
		UIDropDownMenu_AddButton(info)
	end
end

local info_f = CreateFrame("Frame", "QBL_info", UIParent)
info_f:SetPoint("CENTER", 0, 0)
info_f:SetBackdrop(StaticPopup1:GetBackdrop())
info_f:SetHeight(720)
info_f:SetWidth(575)
info_f:Hide()
-- Sets the menu as draggable.
info_f:SetMovable(true)
info_f:SetClampedToScreen(true)
info_f.drag_bar = CreateFrame("Frame", nil, info_f)
info_f.drag_bar:SetSize(450, 24)
info_f.drag_bar:SetPoint("TOP")
info_f.drag_bar:EnableMouse(true)
info_f.drag_bar:RegisterForDrag("LeftButton")
info_f.drag_bar:SetScript("OnDragStart",
	function(self) info_f:StartMoving() end)
info_f.drag_bar:SetScript("OnDragStop",
	function(self) info_f:StopMovingOrSizing() end)
info_f.drag_bar:SetScript("OnHide", function(self)
	try_sound(QBL.sounds.GameDialogClose)
	info_f:StopMovingOrSizing()
end)
-- Creates the menu title border.
info_f.title = CreateFrame("Frame", nil, info_f)
info_f.title:SetPoint("TOP", 0, 1)
info_f.title:SetWidth(200)
info_f.title:SetHeight(24)
info_f.title:SetBackdrop(
	{edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16})
info_f.title:SetBackdropBorderColor(1, 0.82, 0, 0.8)
-- Sets the background for the menu title.
info_f.title_bg = info_f.title:CreateTexture(nil, "BACKGROUND")
info_f.title_bg:SetPoint("TOP", 0, -5)
info_f.title_bg:SetWidth(198)
info_f.title_bg:SetHeight(18)
info_f.title_bg:SetHorizTile(true)
info_f.title_bg:SetVertTile(true)
info_f.title_bg:SetTexture(
	"Interface\\FrameGeneral\\UI-Background-Rock", true, true)
info_f.title_bg:SetVertexColor(1, 1, 1, 1)
-- Sets menu title text.
info_f.title_txt = info_f.title:CreateFontString( "QBL_menu_title", "OVERLAY",
	"GameFontNormal")
info_f.title_txt:SetText("Questionable " .. i18n.INFO)
info_f.title_txt:SetPoint("TOP", 0, -5)
-- Create a box frame to constrain the scrolling content's size.
info_f.box = CreateFrame("Frame", nil, info_f)
info_f.box:SetHeight(640)
info_f.box:SetWidth(525)
info_f.box:SetPoint("TOP", 0, -25)
-- Scroll frame to hold the slider.
info_f.scroll_f = CreateFrame("ScrollFrame", nil, info_f.box)
info_f.scroll_f:SetPoint("TOPLEFT", 10, -10)
info_f.scroll_f:SetPoint("BOTTOMRIGHT", -10, 10)
-- Scroll bar for the slider.
info_f.scroll_bar = CreateFrame("Slider", nil, info_f.scroll_f,
	"UIPanelScrollBarTemplate")
info_f.scroll_bar:SetPoint("TOPLEFT", info_f, "TOPRIGHT", 4, -16)
info_f.scroll_bar:SetPoint("BOTTOMLEFT", info_f, "BOTTOMRIGHT", 4, 16)
info_f.scroll_bar:SetMinMaxValues(1, 200)
info_f.scroll_bar:SetValueStep(1)
info_f.scroll_bar.scrollStep = 1
info_f.scroll_bar:SetValue(0)
info_f.scroll_bar:SetWidth(16)
info_f.scroll_bar:SetScript("OnValueChanged",
	function (self, value) self:GetParent():SetVerticalScroll(value) end)
info_f.scroll_bar:SetBackdrop({
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true})
-- Scrolling conent.
info_f.content = CreateFrame("Frame", nil, info_f.scroll_f)
info_f.content:SetSize(500, 1)
info_f.content_txt = info_f.content:CreateFontString(nil, "BACKGROUND",
	"GameFontNormal")
info_f.content_txt:SetText(i18n.MORE_INFO)
info_f.content_txt:SetPoint("TOP", 0, 0)
info_f.content_txt:SetJustifyH("LEFT")
info_f.content_txt:SetTextColor(0.8, 0.8, 0.8, 1)
info_f.scroll_f:SetScrollChild(info_f.content)
-- Button to launch Q? menu.
info_f.menu_btn = CreateFrame( "Button", "QBL_info_menu_btn", info_f,
	"UIPanelButtonTemplate")
info_f.menu_btn.menu_f = menu_f
info_f.menu_btn:SetPoint("BOTTOM", 0, 20)
info_f.menu_btn:SetPoint("LEFT", 25, 0)
info_f.menu_btn:SetHeight(25)
info_f.menu_btn:SetWidth(100)
info_f.menu_btn:SetText(i18n.MENU)
info_f.menu_btn:SetScript("OnClick",
	function(self)
		HideParentPanel(self)
		self.menu_f:Show()
	end)
-- Prints a GitHub link.
info_f.gh_link = CreateFrame( "Button", nil, info_f)
info_f.gh_link:SetPoint("BOTTOM", 0, 20)
info_f.gh_link:SetPoint("CENTER", 0, 0)
info_f.gh_link:SetHeight(25)
info_f.gh_link:SetWidth(175)
info_f.gh_link:SetScript("OnClick", function(self)
	local editbox = ChatEdit_GetLastActiveWindow()
	ChatEdit_ActivateChat(editbox)
	editbox:Insert("https://github.com/pandasoft/questionable/")
	editbox:HighlightText()
end)
info_f.gh_link_txt = info_f.gh_link:CreateFontString(nil, "OVERLAY", "GameFontNormal")
info_f.gh_link_txt:SetText("https://github.com/pandasoft")
info_f.gh_link_txt:SetPoint("TOP", 0, -5)
info_f.close_btn = CreateFrame( "Button", "QBL_info_close_btn", info_f,
	"UIPanelButtonTemplate")
info_f.close_btn:SetPoint("BOTTOM", 0, 20)
info_f.close_btn:SetPoint("RIGHT", -25, 0)
info_f.close_btn:SetHeight(25)
info_f.close_btn:SetWidth(100)
info_f.close_btn:SetText(CLOSE)
info_f.close_btn:SetScript("OnClick", function(self) HideParentPanel(self) end)

-- menu_f initialization.
menu_f:SetPoint("CENTER", 0, 0)
menu_f:SetBackdrop(StaticPopup1:GetBackdrop())
menu_f:SetHeight(720)
menu_f:SetWidth(575)
menu_f:Hide()
menu_f:SetMovable(true)
menu_f:SetClampedToScreen(true)
menu_f.drag_bar = CreateFrame("Frame", nil, menu_f)
menu_f.drag_bar:SetSize(450, 24)
menu_f.drag_bar:SetPoint("TOP")
menu_f.drag_bar:EnableMouse(true)
menu_f.drag_bar:RegisterForDrag("LeftButton")
menu_f.drag_bar:SetScript("OnDragStart",
	function(self) menu_f:StartMoving() end)
menu_f.drag_bar:SetScript("OnDragStop",
	function(self) menu_f:StopMovingOrSizing() end)
menu_f.drag_bar:SetScript("OnHide", function(self)
	try_sound(QBL.sounds.GameDialogClose)
	menu_f:StopMovingOrSizing()
end)
menu_f:SetScript("OnShow", function(self) try_sound(QBL.sounds.GameDialogOpen) end)
menu_f.title = CreateFrame("Frame", nil, menu_f)
menu_f.title:SetPoint("TOP", 0, 1)
menu_f.title:SetWidth(200)
menu_f.title:SetHeight(24)
menu_f.title:SetBackdrop(
	{edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16})
menu_f.title:SetBackdropBorderColor(1, 0.82, 0, 0.8)
menu_f.title_bg = menu_f.title:CreateTexture(nil, "BACKGROUND")
menu_f.title_bg:SetPoint("TOP", 0, -5)
menu_f.title_bg:SetWidth(198)
menu_f.title_bg:SetHeight(18)
menu_f.title_bg:SetHorizTile(true)
menu_f.title_bg:SetVertTile(true)
menu_f.title_bg:SetTexture(
	"Interface\\FrameGeneral\\UI-Background-Rock", true, true)
menu_f.title_bg:SetVertexColor(1, 1, 1, 1)
menu_f.title_txt = menu_f.title:CreateFontString( "QBL_menu_title", "OVERLAY",
	"GameFontNormal")
menu_f.title_txt:SetText("Questionable " .. i18n.CONFIG)
menu_f.title_txt:SetPoint("TOP", 0, -5)
-- Adds a help reminder.
menu_f.help_txt = menu_f:CreateFontString(nil, "OVERLAY", "GameFontNormal")
menu_f.help_txt:SetText(i18n.HELP)
menu_f.help_txt:SetPoint("BOTTOM", 0, 65)
menu_f.help_txt:SetPoint("CENTER", 0, 0)
-- Adds a close button to the menu.
menu_f.close_btn = CreateFrame( "Button", "QBL_menu_close_btn", menu_f,
	"UIPanelButtonTemplate")
menu_f.close_btn:SetPoint("BOTTOM", 0, 20)
menu_f.close_btn:SetPoint("RIGHT", -30, 0)
menu_f.close_btn:SetHeight(25)
menu_f.close_btn:SetWidth(100)
menu_f.close_btn:SetText(CLOSE)
menu_f.close_btn:SetScript("OnClick", function(self) HideParentPanel(self) end)
-- Adds a defaults button to the menu.
menu_f.defaults_btn = CreateFrame("Button", "QBL_menu_defaults_btn", menu_f,
	"UIPanelButtonTemplate")
menu_f.defaults_btn:SetPoint("BOTTOM", 0, 20)
menu_f.defaults_btn:SetPoint("LEFT", 30, 0)
menu_f.defaults_btn:SetHeight(25)
menu_f.defaults_btn:SetWidth(100)
menu_f.defaults_btn:SetText(DEFAULTS)
menu_f.defaults_btn:SetScript("OnClick", function(self)
	QBL_cfg = QBL:copy_cfg()
	-- 'check' and 'drop' created during init_menu().
	for name, btn in pairs(menu_f.check)
	do btn:SetChecked(QBL_cfg[name]) end
	for name, btn in pairs(menu_f.drop)
	do
		-- If the dropdown is not reinitialized the selection text will be wrong.
		UIDropDownMenu_Initialize(btn, init_dropdown)
		UIDropDownMenu_SetSelectedID(btn, QBL_cfg[name])
	end
	PlaySound(QBL.sounds.InterfaceSoundCharWindowClose)
end)
-- Adds a 'More Info' button.
menu_f.info_btn =  CreateFrame("Button", "QBL_menu_info_btn", menu_f,
	"UIPanelButtonTemplate")
menu_f.info_btn.info_f = info_f
menu_f.info_btn:SetPoint("BOTTOM", 0, 20)
menu_f.info_btn:SetPoint("CENTER", 0, 0)
menu_f.info_btn:SetHeight(25)
menu_f.info_btn:SetWidth(175)
menu_f.info_btn:SetText(i18n.INFO)
menu_f.info_btn:SetScript("OnClick", function(self)
	HideParentPanel(self)
	self.info_f:Show()
end)

-- Make menu frames closable with ESC.
tinsert(UISpecialFrames, "QBL_menu")
tinsert(UISpecialFrames, "QBL_info")

-- Creates and returns a checkbox frame.
-- 'row' is an integer starting from 1 to position the dropdown's row.
-- 'column' is either 'LEFT' or 'RIGHT' to set the dropdown's column.
-- 'option' is the QBL_cfg option.
-- 'label' is a string to caption the dropdown with.
-- 'tooltip' is a string providing the tooltip text for this button.
-- 'onclick' is an optional OnClick handler to use.
local function make_checkbox(row, column, option, label, tooltip, onclick)
	local checkbox = CreateFrame("CheckButton", "QBL_" .. option .. "_btn",
		menu_f, "ChatConfigCheckButtonTemplate")
	checkbox:SetPoint("TOP", 0, -30 * row)
	if column == "LEFT" then checkbox:SetPoint("LEFT", 35, 0)
	elseif column == "RIGHT" then checkbox:SetPoint("RIGHT", -215, 0) end

	getglobal(checkbox:GetName() .. 'Text'):SetText(label)
	if tooltip then checkbox.tooltip = tooltip end
	if onclick then checkbox:SetScript("OnClick", onclick) else
		checkbox:SetScript("OnClick", function(self)
			QBL_cfg[option] = not QBL_cfg[option]

			-- Start and stop services for 'option', if needed.
			if QBL_cfg[option]
				and (QBL.srv.opt[option] or QBL.hook[option])
				and not QBL.srv.on[option]
			then
				QBL:enable_opt(option)
			elseif not QBL_cfg[option]
				and (QBL.srv.opt[option] or QBL.hook[option])
				and QBL.srv.on[option]
			then
				QBL:disable_opt(option)
			end
		end)
	end

	checkbox:SetChecked(QBL_cfg[option])
	return checkbox
end

-- Creates and returns a dropdown frame. Takes up two rows.
-- 'row' is an integer starting from 1 to position the dropdown's row.
-- 'option' is the QBL_cfg option.
-- 'label' is a string to caption the dropdown with.
-- 'items' is a table of strings used to label the dropdown options.
-- 'tip' is a string providing the tooltip text for this button.
-- 'onclick' is an optional OnClick handler.
local function make_dropdown(row, column, option, label, items, tip, onclick)
	local dropdown = CreateFrame("Button", "QBL_" .. option .. "_btn", menu_f,
		"UIDropDownMenuTemplate")
	dropdown:SetPoint("TOP", 0, -30 * row)
	if column == "LEFT" then dropdown:SetPoint("LEFT", 110, -5)
	elseif column == "RIGHT" then dropdown:SetPoint("RIGHT", -15, 0) end

	-- Provides a default handler that starts and stops services for 'option'.
	-- 'self.owner' is set during init_dropdown(), which points it to the
	-- 'dropdown' in this scope.
	if not onclick then onclick = function(self, id, ...)
			-- This event can fire when unrelated parts of the dropdown are
			-- clicked. They do not send a numerical id and can be filtered.
			id = tonumber(id)
			if not id then return end

			QBL_cfg[option] = id

			if id == 1
				and (QBL.srv.opt[option] or QBL.hook[option])
				and QBL.srv.on[option]
			then
				QBL:disable_opt(option)
			elseif id > 1
				and (QBL.srv.opt[option] or QBL.hook[option])
				and not QBL.srv.on[option]
			then
				QBL:enable_opt(option)
			end

			UIDropDownMenu_SetSelectedID(self.owner, id)
		end
	end

	dropdown.txt = dropdown:CreateFontString(nil, "BACKGROUND", "GameFontWhite")
	dropdown.txt:SetText(label)
	dropdown.txt:SetPoint("RIGHT", -160, 5)
	dropdown.items = items
	dropdown.onclick = onclick
	if tip then dropdown.tiptitle, dropdown.tip = label, tip end
	dropdown:SetScript("OnClick", onclick)

	UIDropDownMenu_Initialize(dropdown, init_dropdown)
	UIDropDownMenu_SetWidth(dropdown, 120)
	UIDropDownMenu_SetButtonWidth(dropdown, 124)
	UIDropDownMenu_JustifyText(dropdown, "LEFT")
	UIDropDownMenu_SetSelectedID(dropdown, QBL_cfg[option])
	return dropdown
end

-- Adds 'caption' at 'row'.
local function make_label(row, caption)
	-- Properly offset category label from menu buttons.
	row = (-30 * (row + 1)) + 25

	local label = CreateFrame("Frame", nil, menu_f)
	label:SetPoint("TOP", 0, row)
	label:SetWidth(100)
	label:SetHeight(24)
	-- Creates the label border.
	label:SetBackdrop(
		{edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16})
	label:SetBackdropBorderColor(1, 0.82, 0, 0.8)
	-- Sets the label background.
	label.bg = label:CreateTexture(nil, "ARTWORK")
	label.bg:SetPoint("TOP", 0, -5)
	label.bg:SetWidth(94)
	label.bg:SetHeight(18)
	label.bg:SetHorizTile(true)
	label.bg:SetVertTile(true)
	label.bg:SetTexture(
		"Interface\\FrameGeneral\\UI-Background-Rock", true, true)
	label.bg:SetVertexColor(1, 1, 1, 1)
	-- Creates a blue horizontal line.
	label.bar = label:CreateTexture(nil, "BACKGROUND")
	label.bar:SetPoint("CENTER")
	label.bar:SetWidth(500)
	label.bar:SetHeight(1)
	label.bar:SetColorTexture(0.25, 0.41, 0.88, 1.0)
	-- Sets the label text.
	label.txt = label:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	label.txt:SetText(caption)
	label.txt:SetPoint("TOP", 0, -5)

	return label
end

--[[ Interface Options Menu ]]--

local iface_menu_f = CreateFrame("Frame", "QBL_iface_menu")
iface_menu_f.name = "Questionable"
iface_menu_f.btn = CreateFrame("Button", "QBL_iface_menu_btn", iface_menu_f,
	"OptionsButtonTemplate")
iface_menu_f.btn:SetText(i18n.IFACE_BTN)
iface_menu_f.btn:SetWidth(200)
iface_menu_f.btn:SetHeight(75)
iface_menu_f.btn:SetPoint("CENTER")
iface_menu_f.btn:RegisterForClicks("AnyUp")
iface_menu_f.btn:SetScript("OnClick", function (self)
	InterfaceOptionsFrameCancel_OnClick()
	HideUIPanel(GameMenuFrame)
	menu_f:Show()
end)

InterfaceOptions_AddCategory(iface_menu_f)

--[[ Slash Command ]]--

SLASH_QBL1 = '/qbl'
SLASH_QBL2 = '/questionable'
function SlashCmdList.QBL(msg, editbox)
	if msg == "follow" or msg == "f" then
		if UnitName("target") then
			if IsInGroup()
			then QBL:send_msg("FOL" .. UnitName("target"))
			else SendChatMessage("qbl f", "WHISPER", nil, UnitName("target")) end
		else QBL:print(i18n.NEED_TARGET) end
	elseif msg == "promote" or msg == "p" then
		if IsInGroup()
		then QBL:send_msg("GGL")
		else QBL:print(i18n.NEED_GROUP) end
	elseif msg == "invite" or msg == "i"
		then SendChatMessage("qbl i", "WHISPER", nil, UnitName("target"))
	else menu_f:Show() end
end

--[[ Initialization ]]--

-- Initializes the main menu layout. Called during the ADDON_LOADED event.
function QBL.lib.Interface.init_menu()
	-- Tables for category labels and UI elements.
	-- Used to access menu UI from the Default button and other UI helpers.
	menu_f.cat, menu_f.check, menu_f.drop, menu_f.txt = {}, {}, {}, {}

	menu_f.cat.quest = make_label(1, i18n.QUESTING)
	menu_f.check.tracker = make_checkbox(2, "LEFT", "tracker",
		i18n.SHARE_O, i18n.SHARE_O_TIP)
	menu_f.check.print_quests = make_checkbox(2, "RIGHT", "print_quests",
		i18n.PRINT_Q, i18n.PRINT_Q_TIP)
	menu_f.drop.auto_quests = make_dropdown(3, "LEFT", "auto_quests", i18n.ACCEPT_Q,
		{i18n.OFF, i18n.A_ONLY, i18n.T_ONLY, i18n.T_AND_A},
		i18n.ACCEPT_Q_TIP)
	menu_f.check.accept_trivial = make_checkbox(3, "RIGHT", "accept_trivial",
		i18n.ACCEPT_TQ, i18n.ACCEPT_TQ_TIP)
	menu_f.drop.do_dailies = make_dropdown(4, "LEFT", "do_dailies", i18n.ACCEPT_D,
		{i18n.OFF, i18n.A_ONLY, i18n.T_ONLY, i18n.T_AND_A},
		i18n.ACCEPT_D_TIP)
	menu_f.check.auto_reward = make_checkbox(4, "RIGHT", "auto_reward",
		i18n.SELECT_R, i18n.SELECT_R_TIP)

	menu_f.cat.friends = make_label(5, i18n.FRIENDS)
	menu_f.check.follow_friend = make_checkbox(6, "LEFT", "follow_friend",
		i18n.FOLLOW_F, i18n.FOLLOW_F_TIP)
	menu_f.check.accept_invite = make_checkbox(6, "RIGHT", "accept_invite",
		i18n.ACCEPT_G, i18n.ACCEPT_G_TIP)
	menu_f.check.promote_friend = make_checkbox(7, "LEFT", "promote_friend",
		i18n.PROMOTE_F, i18n.PROMOTE_F_TIP)
	menu_f.check.invite_friend = make_checkbox(7, "RIGHT", "invite_friend",
		i18n.INVITE_F, i18n.INVITE_F_TIP)
	menu_f.check.bnet_friends = make_checkbox(8, "LEFT", "bnet_friends",
		i18n.BNET_F, i18n.BNET_F_TIP)

	menu_f.cat.TRASH = make_label(9, i18n.TRASH)
	menu_f.check.auto_sell = make_checkbox(10, "LEFT", "auto_sell",
		i18n.AUTO_SR, i18n.AUTO_SR_TIP)
	menu_f.check.guild_repair = make_checkbox(10, "RIGHT", "guild_repair",
		i18n.GUILD_R, i18n.GUILD_R_TIP)
	menu_f.check.drop_trash_loot = make_checkbox(11, "LEFT", "drop_trash_loot",
		i18n.REPLACE_T, i18n.REPLACE_T_TIP)
	menu_f.check.drop_trash_reward = make_checkbox(11, "RIGHT", "drop_trash_reward",
		i18n.DROP_T, i18n.DROP_T_TIP)
	menu_f.check.keep_gray = make_checkbox(12, "LEFT", "keep_gray",
		i18n.KEEP_GR, i18n.KEEP_GR_TIP)
	menu_f.check.keep_de = make_checkbox(12, "RIGHT", "keep_de",
		i18n.KEEP_DE, i18n.KEEP_DE_TIP)

	menu_f.cat.loot = make_label(13, i18n.LOOT)
	menu_f.drop.fast_loot = make_dropdown(14, "LEFT", "fast_loot", i18n.FAST_L,
		{i18n.OFF, "|cff9D9D9D" .. i18n.POOR .. "|r", i18n.COMMON,
			"|cff1EFF00" .. i18n.UNCOMMON .. "|r", "|cff0070DD" .. i18n.RARE .. "|r",
			"|cffA335EE" .. i18n.EPIC .. "|r"},
		i18n.FAST_L_TIP)
	menu_f.check.stacker = make_checkbox(14, "RIGHT", "stacker",
		i18n.STACK_B, i18n.STACK_B_TIP)
	menu_f.check.reagent = make_checkbox(15, "LEFT", "reagent",
		i18n.DEPOSIT_RE, i18n.DEPOSIT_RE_TIP)
	menu_f.check.stack_gbank = make_checkbox(15, "RIGHT", "stack_gbank",
		i18n.STACK_GB, i18n.STACK_GB_TIP)
	menu_f.check.loot_left = make_checkbox(16, "LEFT", "loot_left",
		i18n.LEFT_L, i18n.LEFT_L_TIP)

	menu_f.cat.meta = make_label(17, i18n.INTERFACE)
	menu_f.check.world_coord = make_checkbox(18, "LEFT", "world_coord",
		i18n.WORLD_M, i18n.WORLD_M_TIP)
	menu_f.check.mini_coord = make_checkbox(18, "RIGHT", "mini_coord",
		i18n.MINI_M, i18n.MINI_M_TIP)
	menu_f.check.reverse_meta = make_checkbox(19, "LEFT", "reverse_meta",
		i18n.HOLD_META, i18n.HOLD_META_TIP)
	menu_f.check.confirm_de = make_checkbox(19, "RIGHT", "confirm_de",
		i18n.CONFIRM_DE, i18n.CONFIRM_DE_TIP)
	menu_f.check.sound = make_checkbox(20, "LEFT", "sound",
		i18n.NOTICE_S, i18n.NOTICE_S_TIP)
	menu_f.check.intro = make_checkbox(20, "RIGHT", "intro",
		i18n.ENABLE_I, i18n.ENABLE_I_TIP)
end
