local i18n = QBL.i18n[QBL.locale].lib.Maps

-- Export table for Maps.
QBL.lib.Maps = {}

local function update_coords()
	if QBL_cfg.mini_coord and Minimap:IsVisible() then
		local x, y = GetPlayerMapPosition("player")
		if x and y and x ~= 0 and y ~= 0 then
			MinimapZoneText:SetText(
				format("(%d:%d) ", x * 100.0, y * 100.0) .. GetMinimapZoneText())
		else MinimapZoneText:SetText(GetMinimapZoneText()) end
	end

	if QBL_cfg.world_coord and WorldMapFrame:IsVisible() then
		local x, y = GetCursorPosition()
		local px, py = GetPlayerMapPosition("player")
		if x and y and px and py then
			local height = WorldMapDetailFrame:GetHeight()
			local width = WorldMapDetailFrame:GetWidth()
			local scale = WorldMapDetailFrame:GetEffectiveScale()
			local left = WorldMapDetailFrame:GetLeft()
			local top = WorldMapDetailFrame:GetTop()

			x = x / scale
			y = y / scale
			x = ((x - left) / width) * 100
			y = ((top - y) / height) * 100

			local txt = i18n.CURSOR .. ": " .. format("(%.1f:%.1f)", x, y)
			if px ~= 0 and py ~= 0 then
				txt = i18n.PLAYER .. ": "
					.. format("(%.1f:%.1f)", px * 100, py * 100) .. " | " .. txt
			else txt = i18n.PLAYER .. ": ?" .. " | " .. txt end
			WorldMapFrame.BorderFrame.TitleText:SetText(txt)
		else WorldMapFrame.BorderFrame.TitleText:SetText(MAP_AND_QUEST_LOG) end
	end
end

local elapsed = 0
function QBL.lib.Maps.coord_timer(time)
	elapsed = elapsed + time
	if (elapsed > 0.2) then
		elapsed = 0
		update_coords()
	end
end

-- Event handlers.
function QBL.events:ZONE_CHANGED(...) update_coords() end
function QBL.events:ZONE_CHANGED_INDOORS(...) update_coords() end
function QBL.events:ZONE_CHANGED_NEW_AREA(...) update_coords() end

-- Events required by each configurable option used for this library.
QBL.srv.mini_coord = { "ZONE_CHANGED", "ZONE_CHANGED_INDOORS", "ZONE_CHANGED_NEW_AREA" }
QBL.srv.world_coord = { "ZONE_CHANGED", "ZONE_CHANGED_INDOORS", "ZONE_CHANGED_NEW_AREA" }

-- Hook functions to be called each time a configuration option is toggled.
function QBL.hook.mini_coord(enable, frame)
	if enable then
		if not QBL.srv.on.world_coord then
			frame:SetScript("OnUpdate",
				function(self, time) QBL.lib.Maps.coord_timer(time) end)
		end
	else
		if not QBL.srv.on.world_coord then
			frame:SetScript("OnUpdate", nil)
		end
		MinimapZoneText:SetText(GetMinimapZoneText())
	end
end
function QBL.hook.world_coord(enable, frame)
	if enable then
		if not QBL.srv.on.mini_coord then
			frame:SetScript("OnUpdate",
				function(self, time) QBL.lib.Maps.coord_timer(time) end)
		end
	else
		if not QBL.srv.on.mini_coord then
			frame:SetScript("OnUpdate", nil)
		end
		WorldMapFrame.BorderFrame.TitleText:SetText(MAP_AND_QUEST_LOG)
	end
end
